# encoding: utf-8

module RSeed
  class Server < Sinatra::Base

    # ConstrucThor huhu
    # -----------------
    def initialize(*args, &blk)
      super
      RSeed::Config.instance_eval(&blk) if block_given?
      @config = Config
    end

    # Sinatra Conf
    # ------------
    set :root, File.join(File.dirname(__FILE__), '..', '..')
    set :haml, :format => :html5

    before do
      cache_control :public, :must_revalidate, :max_age => 60
    end

    # Helperz
    # -------
    helpers do
      include Rack::Utils
      
      def host
        "http://#{request.host}#{request.port == 80 ? '' : ':' + request.port.to_s}"
      end

      def auth!
        throw(:halt, [401, "Aweurglaweurglglglwargl !\n"]) if not auth?
      end

      def auth?
        @user = request.cookies["user"]
        @token = request.cookies["token"]
        check = Session.valid?(@user, @token)
        if !check
          puts "I think we have a problem with this session : #{@user.inspect}, #{@token.inspect}"
        end
        check
      end
    end

    # Sessions
    # --------
    get '/signin' do
      if auth?
        redirect '/'
      else
        haml :signin
      end
    end

    post '/signin' do
      token = Session.signin(params[:username], params[:password])
      response.set_cookie("user", :value => params[:username], :path => '/', :expires => Time.now + 60*60*24*90)
      response.set_cookie("token", :value => token, :path => '/', :expires => Time.now + 60*60*24*90)
      redirect '/'
    end


    # Index
    # ----- 
    get '/' do
      rtorrent = XMLRPC::Client.new("seed.aonoscantrad.fr", "/RPC2", 80)
      stats = Hash.new
      begin
        results = rtorrent.call("d.multicall", "main", "d.name=", "d.completed_bytes=", "d.size_bytes=", "d.ratio=", "d.up.rate=", "d.down.rate=", "d.hash=")
        down = rtorrent.call("get_download_rate").to_i / 1024
        up = rtorrent.call("system", "upload_rate").to_i / 1024
        stats = { "down" => down, "up" => up }
      rescue XMLRPC::FaultException => e
        puts "Error:"
        puts e.faultCode
        puts e.faultString
      end
      puts "#{stats}"
      haml :index, :locals => { :result => results, :stats => stats }
    end
  end
end
